#include <qcanvas.h>
#include <qstring.h>
#include <qimage.h>
#include <qtooltip.h>
#include <qtimer.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <kwin.h>

#include <time.h>
#include "Face.h"
#include "SkinManager.h"

Face::Face( SkinManager *manager, QCanvas *canvas, QWidget *parent, const char *name )
  : QObject( parent, name ),
    QCanvasSprite( manager->getPixmap(manager->getDefaultSkin()), canvas )
{
  
  currentType = STRAIGHT_AHEAD;
  currentLevel = 0;
  container = canvas;
  timeout = 0;
  setAnimated(TRUE);
  show(); 
  appletView = parent; 
  winmodule = new KWinModule(this);
  random = new KRandomSequence(time(NULL));




  // connecting the face to the desktop events
  currentDesktop = KWin::currentDesktop();
  connect(winmodule,SIGNAL(currentDesktopChanged(int)),this,SLOT(currentDesktopChanged(int)));
  connect(winmodule,SIGNAL(windowAdded(WId)),this,SLOT(windowAdded(WId)));
  connect(winmodule,SIGNAL(windowRemoved(WId)),this,SLOT(windowRemoved(WId)));

  // connecting the face to the CPU load events
  sensorManager= new KSGRD::SensorManager();
  sensorManager->engage("localhost","","ksysguardd");
  cpuTimeout = 0;
  cpuMedia = 0;
  userLoad = 0;
  sysLoad = 0;
  niceLoad = 0;

  // connect menu actions to this 
  skinManager = manager;
  connect(skinManager,SIGNAL(skinChanged(QCanvasPixmapArray *)),this,SLOT(setPixmap(QCanvasPixmapArray *)));
}



void Face::mouseEntered(QEvent *evt) {
  if (currentLevel == 0) {
    changeFace(DOPED,3);
  }
}

void Face::setPixmap(QCanvasPixmapArray *array){
  setSequence(array);
  adjustPosition();
}

void Face::adjustPosition() {
  move((appletView->width()/2)-(image()->width()/2),(appletView->height()/2)-(image()->height()/2),0);
}

void Face::advance(int phase) {
  if (phase == 1) {
    setFrame(faceNumber(currentType,currentLevel));

    // glancing
    if (timeout == 0) {
      if (currentType == GLANCING_LEFT) {
	changeFace(GLANCING_RIGHT,2);
      } else {
	currentType = STRAIGHT_AHEAD;
	if (random->getLong(20) == 1) {
	  changeFace(GLANCING_LEFT,2);
	}
      }
    } else {
      timeout--;
    }
    

    // request the load info
    if (cpuTimeout == 0) {
      sensorManager->sendRequest("localhost","cpu/user",this,666);
      sensorManager->sendRequest("localhost","cpu/sys",this,667);
      sensorManager->sendRequest("localhost","cpu/nice",this,668);
      cpuTimeout = CPUPOLL;
    } else {
      cpuTimeout--;
    }

  }
}

Face::~Face(){
}

void Face::resizeEvent(QResizeEvent *_Event){
  QSize newSize = _Event->size();
  adjustPosition();
  container -> update();
  
}

int Face::faceNumber(int type,int level) {
  if (type == DEAD) {
    return 40;
  }
  else if (type == DOPED) {
    return 41;
  } else {
    return (type*5) + level;
  }
}

// Set the face to be of type "type" for "tics" tics (calls to method advance)

void Face::changeFace(int type,int tics){
  currentType = type;
  timeout = tics;
}

void Face::currentDesktopChanged(int d) {
  if (d != currentDesktop) {
    if (d < currentDesktop) {
      changeFace(LOOKING_LEFT,3);
    } else {
      changeFace(LOOKING_RIGHT,3);
    }
    currentDesktop = d;
  }
}


void Face::answerReceived( int x, const QString& f ){
  if (x == 667) {
    sysLoad = f.toInt();
  } else if (x == 668) {
    niceLoad = f.toInt();
  } else if (x == 666) {
    userLoad = f.toInt();
    cpuMedia = ((cpuMedia * N_POLLS) + userLoad) / (N_POLLS+1); 

    // calculate the level of misery
    currentLevel = cpuMedia / 20;
    QToolTip::add(appletView,"User Load: " + QString::number(userLoad) + "%\nSystem Load: "+QString::number(sysLoad)+"%\nNice Load: " + QString::number(niceLoad)+"%");
    if (cpuMedia<5){
      currentLevel = 0;
    } else if (cpuMedia <15){
      currentLevel = 1;
    } else if (cpuMedia <30){
      currentLevel = 2;
    } else if (cpuMedia <60){
      currentLevel = 3;
    }  else {
      currentLevel = 4;
    }
  } 
}

void Face::windowAdded(WId w){
  changeFace(SURPRISED,4);
}

void Face::windowRemoved(WId w){
  changeFace(GRINING,4);
  }

 
#include "Face.moc"
