#include "Face.h"
#include "SkinManager.h"
#include <stdlib.h>
#include <qstring.h>
#include <kstandarddirs.h>
#include <kdebug.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <iostream.h>

SkinManager::SkinManager(KConfig *c) : QObject() {
  config = c;
  skinsHash = new QDict<QCanvasPixmapArray>();
  skinsList = new QStringList();
  KStandardDirs *stddirs = new KStandardDirs();
  skinsDir = new QDir(stddirs->findResource("data","karmack/skins/"));
  skinsDir->setFilter( QDir::Dirs );
  skinsDir->setSorting( QDir::Name );
  loadSkinNames();
  initMenu();
}

QPopupMenu *SkinManager::getMenu() {
  return menu;
}

void SkinManager::initMenu() {
  menu = new QPopupMenu();
  skinsMenu = new QPopupMenu();
  connect(skinsMenu,SIGNAL(activated(int)),this,SLOT(changeSkin(int)));
  fillMenus();
}

void SkinManager::fillMenus() {
  menu->clear();
  skinsMenu->clear();
  menu->insertItem("Choose Background color...",this,SLOT(changeColor()));
  menu->insertItem("Skin",skinsMenu);
  for ( QStringList::Iterator it = skinsList->begin(); it != skinsList->end(); ++it ) {
    skinsMenu->insertItem(*it);
  }

}

void SkinManager::changeColor(){
  QColor myColor = config->readColorEntry("color",new QColor(24,52,16));
  int result = KColorDialog::getColor( myColor );
  if ( result == KColorDialog::Accepted ) {
    config->writeEntry("color",myColor);
    emit colorChanged(myColor);
  }
}

QColor SkinManager::getDefaultColor(){
  QColor myColor = config->readColorEntry("color",new QColor(24,52,16));
  return myColor;
}

void SkinManager::loadSkinNames(){
  skinsList->clear();
  const QFileInfoList *list = skinsDir->entryInfoList();
  QFileInfoListIterator it( *list );
  QFileInfo *fi;

  while ( (fi = it.current()) != 0 )  {
    if (!fi->isHidden()) {
      kdDebug() << "KARMACK SKIN:" <<  fi->fileName().latin1() << "\n";
      skinsList->append(fi->fileName());
    }
    ++it;
  }
      
}

void SkinManager::changeSkin(int id){
  QString skin = skinsMenu->text(id);
  kdDebug() << "KARMACK CHANGESKIN " << skin << endl;
  QCanvasPixmapArray *array = getPixmap(skin);
  config->writeEntry("skin",skin);
  emit skinChanged(array);
}

SkinManager::~SkinManager(){
  delete skinsHash;
  delete skinsList;
}

QStringList *SkinManager::getSkinNames(){
  return skinsList;
}

QCanvasPixmapArray* SkinManager::getPixmap(QString skin){
  QCanvasPixmapArray *array;
  if ((array = skinsHash->find(skin)) != 0) {
    kdDebug() << "KARMACK, SKIN ALREADY LOADED\n";
    return array;
  } else {
    QString pixmapexp = skinsDir->absPath() + "/" + skin + "/" + "%1.gif";
    kdDebug() << "KARMACK, SKIN NOT LOADED, LOADING " + pixmapexp+ "\n";
    array = new QCanvasPixmapArray(pixmapexp,42);
    skinsHash->insert(skin,array);
    return array;
  }
}

QString SkinManager::getDefaultSkin(){
  QString skin = config->readEntry("skin","doom");
  return skin;
}

#include "SkinManager.moc"
